/* Customization so that all tables have a set width
 *
 * The default uses colgroup tags with percentages, but it doesn't set a
 * definite size for the table so the table's size is an undefined behavior
 */

table.docutils {
    border: 0;
    border-collapse: collapse;
    width: 85%;
}

/* above is the default, this is the override to go back to normal width */
table.full-width-table {
    border: 0;
    border-collapse: collapse;
    width: inherit;
}
